#ifndef SRSLTE_GSM_SMS_H
#define SRSLTE_GSM_SMS_H

extern "C" {
#include <osmocom/gsm/protocol/gsm_04_11.h> // this is needed for gsm0411_utils autoformatting destroies order
#include <osmocom/gsm/gsm0411_utils.h>
#include <osmocom/gsm/gsm48_ie.h>
#include <osmocom/gsm/gsm_utils.h>
}

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <codecvt>
#include <locale>

namespace srslte {

#define SMS_HDR_SIZE 128
#define SMS_TEXT_SIZE 256
#define SMS_ADDRESS_MAX 16

typedef enum {
  DTAP_PD_GROUP_CALL_CONTROL = 0x0,
  DTAP_PD_BROADCAST_CALL_CONTROL,
  DTAP_PD_EPS_SESSION_MANAGEMENT_MESSAGES,
  DTAP_PD_CALL_CONTROL_CALL_RELATED_SS_MESSAGES,
  DTAP_PD_GPRS_TRANSPARENT_TRANSPORT_PROTOCOL,
  DTAP_PD_MOBILITY_MANAGEMENT_MESSAGES,
  DTAP_PD_RADIO_RESOURCES_MANAGEMENT_MESSAGES,
  DTAP_PD_EPS_MOBILITY_MANAGEMENT_MESSAGES,
  DTAP_PD_GPRS_MOBILITY_MANAGEMENT_MESSAGES,
  DTAP_PD_SMS_MESSAGES,
  DTAP_PD_GPRS_SESSION_MANAGEMENT_MESSAGES,
  DTAP_PD_NON_CALL_RELATED_SS_MESSAGES,
  DTAP_PD_LOCATION_SERVICES_SPECIFIED,
  DTAP_PD_UNKNOWN,
  DTAP_PD_RESERVED_FOR_EXTENSION_OF_THE_PD_TO_ONE_OCTET_LENGTH,
  DTAP_PD_TESTS_PROCEDURE,
  DTAP_PD_N_TYPE_ITEMS,
} dtap_pd_t;

typedef enum {
  RP_DATA_MS_TO_NETWORK = 0, // uplink
  RP_DATA_NETWORK_TO_MS,     // downlink
  RP_ACK_MS_TO_NETWORK,
  RP_ACK_NETWORK_TO_MS,
  RP_ERROR_MS_TO_NETWORK,
  RP_ERROR_NETWORK_TO_MS,
  RP_SMMA_MS_TO_NETWORK,
  RP_N_MESSAGE_TYPE_N_ITEMS,
} gsm_rp_message_type_t;

typedef enum {
  DTAP_SMS_MESSAGE_CP_DATA  = 0x01,
  DTAP_SMS_MESSAGE_CP_ACK   = 0x04,
  DTAP_SMS_MESSAGE_CP_ERROR = 0x10,
} dtap_sms_message_type;

typedef enum {
  DTAP_CP_CAUSE_NETWORK_FAILURE                = 0x11,
  DTAP_CP_CAUSE_CONGESTION                     = 0x16,
  DTAP_CP_CAUSE_INVALID_TRANSACTION_IDENTIFIER = 0x51,
  DTAP_CP_CAUSE_SEMANTICALLY_INCORRECT         = 0x5F,
  DTAP_CP_CAUSE_INVALID_MANDATORY_INFO         = 0x60,
  DTAP_CP_CAUSE_MSG_TYPE_NON_EXISTENT          = 0x61,
  DTAP_CP_CAUSE_MSG_NOT_COMPTATIBLE            = 0x62,
  DTAP_CP_CAUSE_INFO_ELEM_NON_EXISTENT         = 0x63,
  DTAP_CP_CAUSE_PROTOCOL_ERR                   = 0x6F,
} dtap_cp_cause_t;

typedef enum {
  TP_VP_NOT_PRESENT = 0,
  TP_ENHANCED_FORMAT,
  TP_RELATIVE_FORMAT,
  TP_ABSOLUTE_FORMAT,
} tp_vpf_t;

// from osmocom-bb
typedef struct {
  unsigned long validity_minutes;
  uint8_t       reply_path;
  uint8_t       status_rep_ind;
  uint8_t       ud_hdr_ind;
  uint8_t       protocol_id;
  uint8_t       mti;
  uint8_t       mms;
  tp_vpf_t      tp_vpf;
  uint8_t       tp_vp;
  uint8_t       tp_rd;
  uint8_t       tp_status_rep_req;
  uint8_t       loop_prevention;
  uint8_t       data_coding_scheme;
  uint8_t       msg_ref;
  char          address[20 + 1]; /* DA LV is 12 bytes max, i.e. 10 bytes
                                  * BCD == 20 bytes string */
  time_t  time;
  uint8_t _user_data_length_override;
  uint8_t user_data[SMS_TEXT_SIZE];
  char    text[SMS_TEXT_SIZE];

  uint8_t tp_fcs;
  uint8_t tp_pi;
  time_t  tp_recv_time;
  uint8_t tp_udl;
  uint8_t tp_ud[200];

  uint8_t rp_cause;
  uint8_t rp_err_diag;
  uint8_t _rp_user_data_length_override;
} gsm_sms_t;

typedef struct {
  gsm_rp_message_type_t message_type;
  uint8_t               reference;
  uint8_t               originator_address[20 + 1];
  uint8_t               destination_address[20 + 1];
  gsm_sms_t             gsm_sms;
  uint8_t               o_addr_meta;
  uint8_t               d_addr_meta;
} rp_data_t;

typedef struct {
  dtap_pd_t             protocol_discriminator;
  bool                  ti_flag;
  uint8_t               tio;
  dtap_sms_message_type message_type;
  rp_data_t             rp_data;
  dtap_cp_cause_t       cp_cause;
} dtap_data_t;

typedef enum {
  TP_VP_ENHANCED_NOT_SPEC = 0x00,
  TP_VP_ENHANCED_RELATIVE_FORMAT,
  TP_VP_ENHANCED_RELATIVE_INT,
  TP_VP_ENHANCED_SEMI_OCTET
} tp_vp_enhanced_vpf_t;

bool decode_dtap_data(uint8_t* ie_ptr, dtap_data_t* dtap_data);

bool decode_rp_data(uint8_t* ie_ptr, rp_data_t* rp_data);

bool decode_gsm340_tpdu_to_sms(uint8_t* tpdu, gsm_sms_t* sms, gsm_rp_message_type_t msg_type);

void decode_scts(uint8_t* scts, time_t* time);

bool gen_validity_period(tp_vpf_t tp_vpf, unsigned long minutes);

uint8_t gsm340_gen_addr(uint8_t* ptr, uint8_t type, uint8_t plan, char* address, int* cnt);

bool encode_tpud_sms_deliver(gsm_sms_t* sms, uint8_t* ptr, uint32_t* written_bytes, int sms_testcase);

bool encode_tpud_sms_submit(gsm_sms_t* sms, uint8_t* ptr, uint32_t* written_bytes);

bool encode_tpud_sms_deliver_report(gsm_sms_t* sms, uint8_t* ptr, uint32_t* written_bytes);

bool encode_rpdu(rp_data_t* rp_data, uint8_t* ptr, uint32_t* written_bytes, int sms_testcase);

bool encode_cp_data(dtap_data_t* dtap_data, uint8_t* ptr, uint32_t* written_bytes, int sms_testcase);

bool encode_string_to_bcd_numbers(uint8_t* address, uint8_t* ptr, int* pos);

} // namespace srslte

#endif // SRSLTE_GSM_SMS_H
