#include "srsepc/hdr/mme/sms.h"

#include <ctime>
#include <iostream>

using namespace srslte;

namespace srsepc {

int cnt = 0;

sms::sms(sms_interface_nas* nas_, srslte::log* log_)
{
  sms_pool = srslte::byte_buffer_pool::get_instance();
  sms_log  = log_;
  nas_p    = nas_;

  zmq_channel* zmq_chan = new zmq_channel();
  zmq_chan->init(sms_log, this);
  zmq_chan->start(DEFAULT_PRIORITY);
}

/*
 *   return 0: decoding failed
 *   return != 0: msg ref
 */
uint8_t sms::decode_nas_uplink_transport(srslte::byte_buffer_t* nas_msg)
{
  srslte::dtap_data_t dtap_data = {};

  if (!decode_dtap_data(nas_msg->msg, &dtap_data)) {
    sms_log->error("SMS decoding failed!\n");
    return 0;
  }

  /* Determine if its CP-DATA, CP-ACK or CP-ERROR  */
  switch (dtap_data.message_type) {
    case DTAP_SMS_MESSAGE_CP_DATA: {
      /* Determine RP message type if its a CP Data packet */
      switch (dtap_data.rp_data.message_type) {
        case RP_DATA_MS_TO_NETWORK:
          sms_log->console("Received SMS.\nSender: %s\nMessage: %s\n",
                           dtap_data.rp_data.gsm_sms.address,
                           dtap_data.rp_data.gsm_sms.text);
          return dtap_data.rp_data.reference;

        case RP_DATA_NETWORK_TO_MS:
          sms_log->info("RP Data (N->MS) not handled!\n");
          break;

        case RP_ACK_MS_TO_NETWORK:
          sms_log->info("RP ACK (MS->N) not handled!\n");
          break;

        case RP_ACK_NETWORK_TO_MS:
          sms_log->info("RP ACK (N->MS) not handled!\n");
          break;

        case RP_ERROR_MS_TO_NETWORK:
          sms_log->info("RP ERROR (MS->N) not handled!\n");
          break;

        case RP_ERROR_NETWORK_TO_MS:
          sms_log->info("RP ERROR (N->MS) not handled!\n");
          break;

        case RP_SMMA_MS_TO_NETWORK:
          sms_log->info("RP SMMA (MS->N) not handled!\n");
          break;

        case RP_N_MESSAGE_TYPE_N_ITEMS:
          break;

        default:
          sms_log->error("Unsupported RP message type: %d\n", dtap_data.rp_data.message_type);
          break;
      }
      return 0;
    }

    case DTAP_SMS_MESSAGE_CP_ACK: {
      break;
    }

    case DTAP_SMS_MESSAGE_CP_ERROR: {
      break;
    }

    default:
      sms_log->error("Can not handle message type: %d\n", dtap_data.message_type);
      break;
  }

  return 0;
}

void sms::make_sms_deliver(dtap_data_t* dtap_data, int sms_testcase)
{
  if (sms_testcase == 0) {
    char    msg[]      = "Deliver123";
    char    num[]      = "+491760000000";  // Number of SMSC the sms is coming from!
    char    orig_num[] = "+4917612345678"; // Number of the Sender!
    uint8_t msg_ref    = 1;
    // uint8_t msg_ref    = 0x01;

    uint8_t usr_data[] = {};
    // uint8_t usr_data[] = {0x05, 0x00, 0x03, 0x01, 0x01, 0x01};

    dtap_data->ti_flag = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;

    dtap_data->tio                    = 0x01;
    dtap_data->protocol_discriminator = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type           = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 0;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    dtap_data->rp_data.reference    = msg_ref;
    memset(dtap_data->rp_data.destination_address, 0, sizeof(dtap_data->rp_data.destination_address));
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, num, strlen(num));

    dtap_data->rp_data.gsm_sms.mti             = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.mms             = 0x00;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind  = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind      = 0x00; // here
    dtap_data->rp_data.gsm_sms.reply_path      = 0x00;

    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;
    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;

    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._user_data_length_override = 0;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 0;
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    // FIXME Missing tp-sctp
    dtap_data->rp_data.gsm_sms.time = std::time(0) - 5; // time now - 5sec
    return;
  }

  if (sms_testcase == 1) { // ORIGINATOR_ADDRESS_TOO_SHORT
    char msg[] = "01";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._user_data_length_override = 9;

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time                          = 946681200;
    dtap_data->rp_data.message_type                          = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                                          = 0x01;
    dtap_data->rp_data.reference                             = msg_ref;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;

    char    o_addr[]                                      =   ""; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));

    return;
  }

  if (sms_testcase == 2) { // INCORRECT_RP_USER_DATA_LENGTH
    char msg[] = "02";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x00;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;
    dtap_data->rp_data.gsm_sms._user_data_length_override = 2;
    // uint8_t usr_data[]                                    = { 0x05,             // uhd_len
    //                                                           0x00,             // iei
    //                                                           0x03,             // len
    //                                                           0x00,             // ref_nr
    //                                                           0x01,             // nr_of_msg
    //                                                           0x01 };           // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    // memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char    orig_num[]                                    =   "+1234567890";     // Oridinating Adress: Number of the
    memset(dtap_data->rp_data.gsm_sms.address, 0, sizeof(dtap_data->rp_data.gsm_sms.address)); // originating
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time                          = 946681200;
    dtap_data->rp_data.message_type                          = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                                          = 0x00;
    dtap_data->rp_data.reference                             = msg_ref;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 24;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 3) { // NO_UDH_BODY_LENGTH_UNEQUAL_USER_DATA_LENGTH
    char msg[] = "03";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x00;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x00,  // uhd_len
                          0x00,  // iei
                          0x00,  // len
                          0x00,  // ref_nr
                          0x00,  // nr_of_msg
                          0x00}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._user_data_length_override = 8;
    char    orig_num[]                                    =   "+1234567890";
    // char orig_num[] = ""; // Oridinating Adress: Number of the Sender!
                          // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 25;
    dtap_data->rp_data.gsm_sms.time                          = 946681200;
    dtap_data->rp_data.message_type                          = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                                          = 0x01;
    dtap_data->rp_data.reference                             = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 4) { // DESTINATION_ADDRESS_NOT_EMPTY
    char msg[] = "04";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    
    dtap_data->rp_data.gsm_sms._user_data_length_override = 9;
    
    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));

    char d_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.destination_address, 0, sizeof(dtap_data->rp_data.destination_address));
    memcpy(dtap_data->rp_data.destination_address, d_addr, strlen(d_addr));
    return;
  }

  if (sms_testcase == 5) { // NO_UDH_DESPITE_SIGNALLED
    char msg[] = "05";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[]                                    = { 0x00 };
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char    orig_num[]                                    =   "+1234567890";     // Oridinating Adress: Number of the
    // Sender!
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 25;
    dtap_data->rp_data.gsm_sms._user_data_length_override = 8;

    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 6) { // UDH_DESPITE_NOT_SIGNALLED
    char msg[] = "06";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x00;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x00,  // uhd_len
                          0x00,  // iei
                          0x00,  // len
                          0x00,  // ref_nr
                          0x00,  // nr_of_msg
                          0x00}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char    orig_num[]                                    =   "+1234567890";     // Oridinating Adress: Number of the
    // Sender!
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 25;
    dtap_data->rp_data.gsm_sms._user_data_length_override = 8;
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 7) { // UCS2_INCORRECT_USER_DATA_LENGTH
    char msg[] = "7";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x08;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time                          = 946681200;
    dtap_data->rp_data.message_type                          = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                                          = 0x01;
    dtap_data->rp_data.reference                             = msg_ref;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 8) { // 8BIT_INCORRECT_USER_DATA_LENGTH
    char msg[] = "08";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x04;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;
    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 9) { // GSM7BIT_INCORRECT_USER_DATA_LENGTH
    char msg[] = "09";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 25;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 8;
    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 10) { // IE0_WITH_NUMBER_OF_MESSAGES_ZERO  ??
    char msg[] = "10";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x00,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;
    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 11) { // INCORRECT_UDH_LENGTH
    char msg[] = "11";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x00,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;
    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 12) { // IE0_WITH_SEQUENCE_NUMBER_ZERO
    char msg[] = "12";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x00}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;
    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 13) { // IE0_WITH_SEQUENCE_NUMBER_GREATER_NUMBER_OF_MESSAGES
    char msg[] = "13";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                                       = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                                           = 0x00;
    dtap_data->protocol_discriminator                        = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                                  = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention               = 0x00;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x02}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }

  if (sms_testcase == 14) { // TIMESTAMP_WITH_INVALID_DIGITS
    char msg[] = "14";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    // dtap_data->rp_data.gsm_sms.time                       =   946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 15) { // TIMESTAMP_WITH_MONTH_ZERO  ??
    char msg[] = "15";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    // dtap_data->rp_data.gsm_sms.time                       =   946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 16) { // UCS2_ODD_BODY_LENGTH   ??
    char msg[] = "16";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme            = 0x08;
    dtap_data->rp_data.gsm_sms.reply_path                    = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind                = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind                    = 0x01;
    dtap_data->rp_data.gsm_sms.mms                           = 0x00;
    dtap_data->rp_data.gsm_sms.mti                           = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id                   = 0x00;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x01,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 17) { // CONFLICT_MESSAGE_EXISTS
    char msg[] = "17";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    uint8_t usr_data[] = {0x05,  // uhd_len
                          0x00,  // iei
                          0x03,  // len
                          0x00,  // ref_nr
                          0x02,  // nr_of_msg
                          0x01}; // seq_nr
    memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
    memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    if (cnt == 0) {
      char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
      memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
      memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    } else {
      char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
      memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
      memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    }
    return;
  }
  if (sms_testcase == 18) { // CONFLICT_NUMBER_OF_MESSAGES_CHANGED
    char msg[] = "18";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme            = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path                    = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind                = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind                    = 0x01;
    dtap_data->rp_data.gsm_sms.mms                           = 0x00;
    dtap_data->rp_data.gsm_sms.mti                           = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id                   = 0x00;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    if (cnt == 0) {                
      uint8_t usr_data[] = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x01}; // seq_nr
      cnt++;

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
      dtap_data->rp_data.reference = 1;
    } else {
      uint8_t usr_data[] = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x03,  // nr_of_msg
                            0x02}; // seq_nr

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
      dtap_data->rp_data.reference  = 2;
    };

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 19) { // INCORRECTLY_CALCULATED_MESSAGE_REFERENCE
    char msg[] = "19";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    dtap_data->rp_data.gsm_sms.mms                = 0x00;
    dtap_data->rp_data.gsm_sms.mti                = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id        = 0x00;

    if (cnt == 0) {
      uint8_t usr_data[] = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x01}; // seq_nr
      cnt++;

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    } else {
      uint8_t usr_data[] = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x02}; // seq_nr

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    };

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 20) { // CONFLICT_MORE_MESSAGES_TO_SEND
    char msg[] = "20";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    dtap_data->rp_data.gsm_sms.reply_path         = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind     = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind         = 0x01;
    // dtap_data->rp_data.gsm_sms.mms                        =   0x00;
    dtap_data->rp_data.gsm_sms.mti                           = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id                   = 0x00;
    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    if (cnt == 0) {
      dtap_data->rp_data.gsm_sms.mms = 0x00;
      uint8_t usr_data[]             = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x01}; // seq_nr
      cnt++;

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
      dtap_data->rp_data.reference    = 1;
    } else {
      dtap_data->rp_data.gsm_sms.mms = 0x01;
      uint8_t usr_data[]             = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x02}; // seq_nr

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
      dtap_data->rp_data.reference    = 2;
    };

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;


    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 21) { // CONFLICT_REPLY_PATH
    char msg[] = "21";
    memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
    memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));

    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
    // dtap_data->rp_data.gsm_sms.reply_path                 =   0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind     = 0x01;
    dtap_data->rp_data.gsm_sms.mms            = 0x00;
    dtap_data->rp_data.gsm_sms.mti            = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id    = 0x00;

    dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    if (cnt == 0) {
      dtap_data->rp_data.gsm_sms.reply_path = 0x00;
      uint8_t usr_data[]                    = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x01}; // seq_nr
      cnt++;

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    } else {
      dtap_data->rp_data.gsm_sms.reply_path = 0x01;
      uint8_t usr_data[]                    = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x02}; // seq_nr

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
    };

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;
    uint8_t msg_ref                 = 0x01;
    dtap_data->rp_data.reference    = msg_ref;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  if (sms_testcase == 22) { // CONFLICT_ALPHABET
    dtap_data->ti_flag                         = LIBLTE_MME_TI_FLAG_SENT_FROM_ORIGINATOR;
    dtap_data->tio                             = 0x00;
    dtap_data->protocol_discriminator          = DTAP_PD_SMS_MESSAGES;
    dtap_data->message_type                    = DTAP_SMS_MESSAGE_CP_DATA;
    dtap_data->rp_data.gsm_sms.loop_prevention = 0x00;

    // dtap_data->rp_data.gsm_sms.data_coding_scheme         =   0x00;
    dtap_data->rp_data.gsm_sms.reply_path                    = 0x00;
    dtap_data->rp_data.gsm_sms.status_rep_ind                = 0x00;
    dtap_data->rp_data.gsm_sms.ud_hdr_ind                    = 0x01;
    dtap_data->rp_data.gsm_sms.mms                           = 0x00;
    dtap_data->rp_data.gsm_sms.mti                           = 0x00; // MTI TS 23.040 - 9.2.3.1, 0x00 = sms-deliver
    dtap_data->rp_data.gsm_sms.protocol_id                   = 0x00;

    dtap_data->rp_data.gsm_sms._user_data_length_override    = 9;

    if (cnt == 0) {
      char msg[] = "22";
      memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
      memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));
      dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 26;
      dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x00;
      uint8_t usr_data[]                            = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x01}; // seq_nr
      cnt++;

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
      dtap_data->rp_data.reference    = 1;
    } else {
      char msg[] = "b";
      memset(dtap_data->rp_data.gsm_sms.text, 0, sizeof(dtap_data->rp_data.gsm_sms.text));
      memcpy(dtap_data->rp_data.gsm_sms.text, msg, strlen(msg));
      dtap_data->rp_data.gsm_sms._rp_user_data_length_override = 25;
      dtap_data->rp_data.gsm_sms.data_coding_scheme = 0x08;
      uint8_t usr_data[]                            = {0x05,  // uhd_len
                            0x00,  // iei
                            0x03,  // len
                            0x00,  // ref_nr
                            0x02,  // nr_of_msg
                            0x02}; // seq_nr

      memset(dtap_data->rp_data.gsm_sms.user_data, 0, sizeof(dtap_data->rp_data.gsm_sms.user_data));
      memcpy(dtap_data->rp_data.gsm_sms.user_data, usr_data, sizeof(usr_data));
      dtap_data->rp_data.reference    = 2;

      dtap_data->rp_data.gsm_sms._user_data_length_override    = 8;
    };

    char orig_num[] = "+1234567890"; // Oridinating Adress: Number of the Sender!
    memset(dtap_data->rp_data.gsm_sms.address,
           0,
           sizeof(dtap_data->rp_data.gsm_sms.address)); // originating address for deliver msg
    memcpy(dtap_data->rp_data.gsm_sms.address, orig_num, strlen(orig_num));

    dtap_data->rp_data.gsm_sms.time = 946681200;
    dtap_data->rp_data.message_type = RP_DATA_NETWORK_TO_MS;

    char o_addr[] = "+1234567890"; // Number of SMSC the sms is coming from!
    memset(dtap_data->rp_data.originator_address, 0, sizeof(dtap_data->rp_data.originator_address));
    memcpy(dtap_data->rp_data.originator_address, o_addr, strlen(o_addr));
    return;
  }
  return;
}

// make sms and send it over nas layer
bool sms::epc_send_sms(int sms_testcase)
{
  // make sms
  srslte::byte_buffer_t* nas_dl_sms = sms_pool->allocate();
  srslte::dtap_data_t    dtap_data;

  make_sms_deliver(&dtap_data, sms_testcase);

  if (!encode_cp_data(&dtap_data, nas_dl_sms->msg, &nas_dl_sms->N_bytes, sms_testcase)) {
    sms_log->error("Encoding NAS message failed!\n");
    return false;
  }

  // for debugging
  for (unsigned int i = 0; i < nas_dl_sms->N_bytes; i++) {
    printf("%02X ", nas_dl_sms->msg[i]);
  }
  printf("\n");

  if (!nas_p->send_ota_sms(nas_dl_sms)) {
    sms_log->error("Sending SMS failed!\n");
    sms_pool->deallocate(nas_dl_sms);
    return false;
  };

  sms_log->console("SMS sent!\n");

  // cleanup
  sms_pool->deallocate(nas_dl_sms);
  return false;
}

/* Not tested yet! */
void sms::make_rp_ack(uint8_t* ack, uint8_t ref)
{
  dtap_data_t dtap_data;
  uint32_t    wb                   = 0;
  dtap_data.tio                    = 0x00;
  dtap_data.ti_flag                = true;
  dtap_data.protocol_discriminator = DTAP_PD_SMS_MESSAGES;
  dtap_data.message_type           = DTAP_SMS_MESSAGE_CP_DATA;
  dtap_data.rp_data.message_type   = RP_ACK_NETWORK_TO_MS;
  dtap_data.rp_data.reference      = ref;

  encode_cp_data(&dtap_data, ack, &wb, 0);
}

} // namespace srsepc
