#!/usr/bin/python3

import zmq
import sys
import threading
import time

from datetime import datetime


def tprint(msg):
    """like print, but won't get newlines confused with multiple threads"""
    sys.stdout.write(msg + "\n")
    sys.stdout.flush()


class ServerTask(threading.Thread):
    """ServerTask"""

    def __init__(self):
        threading.Thread.__init__(self)
        self.clients = []

    def run(self):
        self.context = zmq.Context()
        frontend = self.context.socket(zmq.ROUTER)
        frontend.bind("tcp://*:5570")

        backend = self.context.socket(zmq.DEALER)
        backend.bind("inproc://backend")

        workers = []
        for i in range(5):
            worker = ServerWorker(self.context, self.clients)
            worker.start()
            workers.append(worker)

        zmq.proxy(frontend, backend)

        frontend.close()
        backend.close()
        self.context.term()

    def send_to_all(self, msg):
        worker = self.context.socket(zmq.DEALER)
        worker.connect("inproc://backend")
        print(self.clients)
        for client in self.clients:
            worker.send_multipart([client, msg])


class ServerWorker(threading.Thread):
    """ServerWorker"""

    def __init__(self, context, clients):
        threading.Thread.__init__(self)
        self.context = context
        self.clients = clients

    def run(self):
        worker = self.context.socket(zmq.DEALER)
        worker.connect("inproc://backend")
        tprint("Worker started")
        while True:
            ident, msg = worker.recv_multipart()
            # TODO Register protocol here. eNodeB ID and zmq ident mapping
            if ident not in self.clients:
                self.clients.append(ident)
            tprint("Worker received %s from %s" % (msg, ident))
            #  msg = bytes.fromhex('AABBCCDD')
            # print(msg)
            # worker.send_multipart([ident, msg])

        worker.close()


def main():
    """main function"""
    server = ServerTask()
    server.start()
    time.sleep(10)

    while True:
        print(
            "Set SMS case and press enter to send a SMS to all connected UEs...\n 1 : ORIGINATOR_ADDRESS_TOO_SHORT\n 2 : INCORRECT_RP_USER_DATA_LENGTH\n 3 : NO_UDH_BODY_LENGTH_UNEQUAL_USER_DATA_LENGTH\n 4 : DESTINATION_ADDRESS_NOT_EMPTY\n 5 : NO_UDH_DESPITE_SIGNALLED\n 6 : UDH_DESPITE_NOT_SIGNALLED\n 7 : UCS2_INCORRECT_USER_DATA_LENGTH\n 8 : 8BIT_INCORRECT_USER_DATA_LENGTH\n 9 : GSM7BIT_INCORRECT_USER_DATA_LENGTH\n 10 : IE0_WITH_NUMBER_OF_MESSAGES_ZERO\n 11 : INCORRECT_UDH_LENGTH\n 12 : IE0_WITH_SEQUENCE_NUMBER_ZERO\n 13 : IE0_WITH_SEQUENCE_NUMBER_GREATER_NUMBER_OF_MESSAGES\n 14 : TIMESTAMP_WITH_INVALID_DIGITS\n 15 : TIMESTAMP_WITH_MONTH_ZERO\n 16 : UCS2_ODD_BODY_LENGTH\n 17 : CONFLICT_MESSAGE_EXISTS (2x)\n 18 : CONFLICT_NUMBER_OF_MESSAGES_CHANGED (2x)\n 19 : INCORRECTLY_CALCULATED_MESSAGE_REFERENCE (2x)\n 20 : CONFLICT_MORE_MESSAGES_TO_SEND (2x)\n 21 : CONFLICT_REPLY_PATH (2x)\n 22 : CONFLICT_ALPHABET (2x)\n"
        )
        sms_case = input()
        # get time stamp for debugging
        now = datetime.now()
        current_time = now.strftime("%H:%M:%S")
        print("Current Time =", current_time)
        server.send_to_all(sms_case.encode("utf-8"))
    server.join()


if __name__ == "__main__":
    main()
